//**************************************************************************//
//**                                                                      **//
//**                            DebugForm                                 **//
//**                                                                      **//
//**   Copyright (C) Dott. Cosimo De Michele 1998                         **//
//**         You are not allowed to distribuite this software             **//
//**         commercially. Please, notify me, if you make any             **//
//**         changes to this file.                                        **//
//**                                                                      **//
//**************************************************************************//

unit DebugForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Menus, Buttons, ExtCtrls, MF6502;

const
  NO_DEBUG_ITEM = 18;

  LST_NONE = 0;
  LST_STOP = 1;
  LST_POINTER = 2;

  C_FLAG = $01; // Carry occurred
  Z_FLAG = $02; // Result is zero
  I_FLAG = $04; // Interrupts disabled
  D_FLAG = $08; // Decimal mode
  B_FLAG = $10; // Break [0 on stack after int]
  R_FLAG = $20; // Always 1
  V_FLAG = $40; // Overflow occured
  N_FLAG = $80; // Result is negative

type
  TfrmDebug = class(TForm)
    ImageList: TImageList;
    StatusBar: TStatusBar;
    lblA: TLabel;
    lblX: TLabel;
    lblY: TLabel;
    lblS: TLabel;
    lblPC: TLabel;
    gbP: TGroupBox;
    edA: TEdit;
    edX: TEdit;
    edY: TEdit;
    edS: TEdit;
    edPC: TEdit;
    cbC: TCheckBox;
    cbZ: TCheckBox;
    cbI: TCheckBox;
    cbD: TCheckBox;
    cbB: TCheckBox;
    cbR: TCheckBox;
    cbV: TCheckBox;
    cbN: TCheckBox;
    Panel1: TPanel;
    spRun: TSpeedButton;
    MainMenu1: TMainMenu;
    Debug1: TMenuItem;
    spPause: TSpeedButton;
    spStep: TSpeedButton;
    DebugList: TListView;
    lblInterrupt: TLabel;
    spSkip: TSpeedButton;
    mnuView: TMenuItem;
    mnuPattern: TMenuItem;
    mnuName: TMenuItem;
    mnuPalette: TMenuItem;
    mnuWave: TMenuItem;
    Bevel1: TBevel;
    procedure FormShow(Sender: TObject);
    procedure StepOver1Click(Sender: TObject);
    procedure cbCClick(Sender: TObject);
    procedure edAChange(Sender: TObject);
    procedure edXChange(Sender: TObject);
    procedure Run1Click(Sender: TObject);
    procedure Pause1Click(Sender: TObject);
    procedure Reset1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Breakpoints1Click(Sender: TObject);
    procedure DebugListDblClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuPatternClick(Sender: TObject);
  private
    { Private declarations }
    LastDebugAddr : Word;
    RunStop : boolean;
    PreviousItem : TListItem;
    BreakPointsList : TStringList;
    procedure GotoAddress(addr: word);
    procedure LoadPage(addr: word);
    procedure AlignRegister;
    procedure AlignItem(l : TListItem);
  public
    { Public declarations }
  end;

  function DAsm(var addr : word) : string;
  function ExecCPU(var R: M6502): word;

var
  frmDebug: TfrmDebug;

implementation

uses
  NesForm;

{$R *.DFM}

procedure TfrmDebug.FormCreate(Sender: TObject);
begin
  BreakPointsList := TStringList.Create;
  PreviousItem := nil;
end;

procedure TfrmDebug.FormDestroy(Sender: TObject);
begin
  BreakPointsList.free;
end;

procedure TfrmDebug.AlignRegister;
begin
  Tag := 1;
  edA.Text  := IntToHex(frmNES.cpu.A, 2);
  edX.Text  := IntToHex(frmNES.cpu.X, 2);
  edY.Text  := IntToHex(frmNES.cpu.Y, 2);
  edPC.Text := IntToHex(frmNES.cpu.PC.W, 4);
  edS.Text  := IntToHex(frmNES.cpu.S, 4);

  cbC.Checked := frmNES.cpu.P and C_FLAG <> 0;
  cbZ.Checked := frmNES.cpu.P and Z_FLAG <> 0;
  cbI.Checked := frmNES.cpu.P and I_FLAG <> 0;
  cbD.Checked := frmNES.cpu.P and D_FLAG <> 0;
  cbB.Checked := frmNES.cpu.P and B_FLAG <> 0;
  cbR.Checked := frmNES.cpu.P and R_FLAG <> 0;
  cbV.Checked := frmNES.cpu.P and V_FLAG <> 0;
  cbN.Checked := frmNES.cpu.P and N_FLAG <> 0;
  Tag := 0;
end; { AlignRegister }

procedure TfrmDebug.AlignItem(l : TListItem);
begin
  if Assigned(PreviousItem) then
  begin
    if BreakPointsList.IndexOf(PreviousItem.Caption)=-1 then
      PreviousItem.ImageIndex := LST_NONE
    else
      PreviousItem.ImageIndex := LST_STOP;
    PreviousItem.StateIndex := -1;
  end;
  if Assigned(l) then
  begin
    l.Selected := true;
    //l.ImageIndex := LST_POINTER;
    if BreakPointsList.IndexOf(l.Caption)=-1 then
      l.ImageIndex := LST_NONE
    else
      l.ImageIndex := LST_STOP;
    l.StateIndex := LST_POINTER;
    l.MakeVisible(false);
  end;
end; { AlignItem }

procedure TfrmDebug.GotoAddress(Addr: word);
var
  l : TListItem;
begin
  l := DebugList.FindCaption(1, IntToHex(Addr, 4), false, true, false);
  if Assigned(l) then
    AlignItem(l)
  else
    LoadPage(Addr);
end; { GotoAddress }

procedure TfrmDebug.LoadPage(addr: word);
var
  i : integer;
  l : TListItem;
  TheAddr : word;
begin
  DebugList.Items.BeginUpdate;
  TheAddr := addr;
  for i:=1 to NO_DEBUG_ITEM do
  begin
    l := DebugList.FindCaption(1, IntToHex(Addr, 4), false, true, false);
    if not Assigned(l) then
    begin
      l := DebugList.Items.Add;
      l.Caption := IntToHex(addr, 4);
      l.SubItems.Add(DAsm(addr));
    end;
  end;
  DebugList.Items.EndUpdate;
  LastDebugAddr := addr;
  l := DebugList.FindCaption(0, IntToHex(TheAddr, 4), false, true, false);
  AlignItem(l);
end; { LoadPage }

procedure TfrmDebug.FormShow(Sender: TObject);
begin
  RunStop := false;
  AlignRegister;
  LastDebugAddr := frmNES.cpu.PC.W;
  LoadPage(LastDebugAddr);
end;

procedure TfrmDebug.StepOver1Click(Sender: TObject);
var
  addr : word;
begin
  RunStop := false;
  addr := frmNES.cpu.PC.W;
  PreviousItem := DebugList.FindCaption(0, IntToHex(addr, 4), false, true, false);
  GotoAddress(ExecCPU(frmNES.cpu));
  AlignRegister;
end;

procedure TfrmDebug.Run1Click(Sender: TObject);
var
  addr : word;
begin
  RunStop := false;
  DebugList.Cursor := crHourglass;
  DebugList.Items.BeginUpdate;
  Repeat
    addr := frmNES.cpu.PC.W;
    PreviousItem := DebugList.FindCaption(0, IntToHex(addr, 4), false, true, false);
    GotoAddress(ExecCPU(frmNES.cpu));
    AlignRegister;
    Application.ProcessMessages;
    { Controllo se sul successivo c' un breakpoint }
    RunStop := (BreakPointsList.IndexOfObject(DebugList.Selected)<>-1) or RunStop;
  until RunStop;
  DebugList.Items.EndUpdate;
  DebugList.Cursor := crDefault;
  if Assigned(DebugList.Selected) then
    DebugList.Selected.MakeVisible(false);
end;

procedure TfrmDebug.cbCClick(Sender: TObject);
begin
  if Tag=1 then Exit;
  if TCheckBox(Sender).Name = 'cbC' then
  begin
    if cbC.Checked then
      frmNES.cpu.P := frmNES.cpu.P or C_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor C_FLAG;
  end
  else if TCheckBox(Sender).Name = 'cbZ' then
  begin
    if cbZ.Checked then
      frmNES.cpu.P := frmNES.cpu.P or Z_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor Z_FLAG;
  end
  else if TCheckBox(Sender).Name = 'cbI' then
  begin
    if cbI.Checked then
      frmNES.cpu.P := frmNES.cpu.P or I_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor I_FLAG;
  end
  else if TCheckBox(Sender).Name = 'cbD' then
  begin
    if cbD.Checked then
      frmNES.cpu.P := frmNES.cpu.P or D_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor D_FLAG;
  end
  else if TCheckBox(Sender).Name = 'cbB' then
  begin
    if cbB.Checked then
      frmNES.cpu.P := frmNES.cpu.P or B_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor B_FLAG;
  end
  else if TCheckBox(Sender).Name = 'cbR' then
  begin
    if cbR.Checked then
      frmNES.cpu.P := frmNES.cpu.P or R_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor R_FLAG;
  end
  else if TCheckBox(Sender).Name = 'cbV' then
  begin
    if cbV.Checked then
      frmNES.cpu.P := frmNES.cpu.P or V_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor V_FLAG;
  end
  else if TCheckBox(Sender).Name = 'cbN' then
  begin
    if cbN.Checked then
      frmNES.cpu.P := frmNES.cpu.P or N_FLAG
    else
      frmNES.cpu.P := frmNES.cpu.P xor N_FLAG;
  end
end;

procedure TfrmDebug.edAChange(Sender: TObject);
begin
  if Tag=1 then Exit;
  frmNES.cpu.A := byte(StrToIntDef(edA.Text, 0));
end;

procedure TfrmDebug.edXChange(Sender: TObject);
begin
  if Tag=1 then Exit;
  frmNES.cpu.X := byte(StrToIntDef(edX.Text, 0));
end;

procedure TfrmDebug.Pause1Click(Sender: TObject);
begin
  RunStop := true;
end;

procedure TfrmDebug.Reset1Click(Sender: TObject);
var
  addr: word;
begin
  RunStop := true;
  frmNES.ResetNES;
  addr := frmNES.cpu.PC.W;
  PreviousItem := DebugList.FindCaption(0, IntToHex(addr, 4), false, true, false);
  GotoAddress(ExecCPU(frmNES.cpu));
  AlignRegister;
end;

procedure TfrmDebug.Breakpoints1Click(Sender: TObject);
var
  k : integer;
begin
  if Assigned(DebugList.Selected) then
  begin
    if BreakPointsList.Find(DebugList.Selected.Caption, k) then
    begin
      DebugList.Selected.ImageIndex := LST_NONE;
      BreakPointsList.Delete(k);
    end
    else
    begin
      DebugList.Selected.ImageIndex := LST_STOP;
      BreakPointsList.AddObject(DebugList.Selected.Caption, DebugList.Selected);
    end;
  end;
end;

procedure TfrmDebug.DebugListDblClick(Sender: TObject);
var
  t : TListItem;
begin
  t := DebugList.Selected; //GetItemAt(CurrentX, CurrentY);
  if Assigned(t) then
  begin
    frmNES.cpu.PC.W := Word(StrToIntDef('$' + t.Caption, frmNES.cpu.PC.W));
    AlignRegister;
  end;
end;

procedure TfrmDebug.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  RunStop := true;
  //frmNes.cpu.Run;
end;

procedure TfrmDebug.mnuPatternClick(Sender: TObject);
{var
  frmPatternTable : TfrmPatternTable;}
begin
  {Application.CreateForm(TfrmPatternTable, frmPatternTable);
  frmPatternTable.ShowModal;
  frmPatternTable.free;}
end;

function ExecCPU(var R: M6502): word;
var
  I: byte;
begin
  Result := Exec6502(R);
  { If cycle counter expired... }
  if(R.ICount<=0) then
  begin
    { If we have come after CLI, get INT_? from IRequest }
    {  Otherwise, get it from the loop handler           }
    if (R.AfterCLI)<>0 then
    begin
      I := R.IRequest;            { Get pending interrupt     }
      inc(R.ICount, R.IBackup-1); { Restore the ICount        }
      R.AfterCLI := 0;            { Done with AfterCLI state  }
    end
    else
    begin
      I := Loop6502(R);           { Call the periodic handler }
      R.ICount := R.IPeriod;      { Reset the cycle counter   }
    end;
    if (I=INT_QUIT) then
      Result := R.PC.W; { Exit if INT_QUIT  }
    if (I<>0) then
      Int6502(R, I);    { Interrupt if needed  }
  end;
end;

function MemoryReadWord(addr: word): word;
var
  a1, a2 : byte;
begin
  a1 := frmNES.MemoryReadByte(Addr);
  a2 := frmNES.MemoryReadByte(Addr + 1);
  Result := a1 + (a2 shl 8);
end; { MemoryReadWord }

function DAsm(var addr : word) : string;
var
  s : string;
  opcode : byte;
  c : word;
begin
 with frmNES do
 begin
   opcode := MemoryReadOpcode(addr);
   case opcode of
    $00: s := 'BRK';
    $01: begin
           system.inc(addr);
           s := 'ORA ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
         end;
    $05: begin
           system.inc(addr);
           s := 'ORA $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $09: begin
           system.inc(addr);
           s := 'ORA #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $0D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ORA $' + IntToHex(c, 4);
         end;
    $11: begin
           system.inc(addr);
           s := 'ORA ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $12,
    $15: begin
           system.inc(addr);
           s := 'ORA $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $19: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ORA $' + IntToHex(c, 4) + ', Y';
         end;
    $1D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ORA $' + IntToHex(c, 4) + ', X';
         end;
    $02,$03,$07,$0B,$0F,$13,$17,$1B,$1F,$22,$23,$27,$2B,$2F,$33,$37,$3B,$3F,
    $42,$43,$44,$47,$4B,$4F,$53,$54,$57,$5B,$5C,$5F,$62,$63,$67,$6B,$6F,$73,
    $77,$7B,$7F,$82,$83,$87,$8B,$8F,$93,$97,$9B,$9F,$A3,$A7,$AB,$AF,$B3,$B7,
    $BB,$BF,$C2,$C3,$C7,$CB,$CF,$D3,$D4,$D7,$DB,$DC,$DF,$E2,$E3,$E7,$EA,$EB,
    $EF,$F3,$F4,$F7,$FB,$FC,$FF: s := 'NOP';
    $04,
    $0C: s := 'TSB';
    $06: begin
           system.inc(addr);
           s := 'ASL ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $0E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ASL ' + IntToHex(c, 4);
         end;
    $16: begin
           system.inc(addr);
           s := 'ASL ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $1E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ASL ' + IntToHex(c, 4) + ', X';
         end;
    $08: s := 'PHP';
    $0A: s := 'ASL A';
    $10: begin
           system.inc(addr);
           s := 'BPL ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $14,
    $1C: s := 'TRB';
    $18: s := 'CLC';
    $1A: s := 'INA';
    $20: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'JSR $' + IntToHex(c, 4);
         end;
    $21: begin
           system.inc(addr);
           s := 'AND (' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
         end;
    $25: begin
           system.inc(addr);
           s := 'AND ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $29: begin
           system.inc(addr);
           s := 'AND #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $2D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'AND ' + IntToHex(c, 4);
         end;
    $31: begin
           system.inc(addr);
           s := 'AND (' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $32,
    $35: begin
           system.inc(addr);
           s := 'AND ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $39: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'AND ' + IntToHex(c, 4) + ', Y';
         end;
    $3D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'AND ' + IntToHex(c, 4) + ', X';
         end;
    $24: begin
           system.inc(addr);
           s := 'BIT ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $2C: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'BIT ' + IntToHex(c, 4);
         end;
    $34,
    $3C,
    $89: s := 'BIT';
    $26: begin
           system.inc(addr);
           s := 'ROL $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $2E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ROL $' + IntToHex(c, 4);
         end;
    $36: begin
           system.inc(addr);
           s := 'ROL $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $3E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ROL $' + IntToHex(c, 4) + ', X';
         end;
    $28: s := 'PLP';
    $2A: s := 'ROL A';
    $30: begin
           system.inc(addr);
           s := 'BMI ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $38: s := 'SEC';
    $3A: s := 'DEA';
    $40: s := 'RTI';
    $41: begin
           system.inc(addr);
           s := 'EOR ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
         end;
    $45: begin
           system.inc(addr);
           s := 'EOR $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $49: begin
           system.inc(addr);
           s := 'EOR #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $4D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'EOR $' + IntToHex(c, 4);
         end;
    $51: begin
           system.inc(addr);
           s := 'EOR ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $52,
    $55: begin
           system.inc(addr);
           s := 'EOR $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $59: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'EOR $' + IntToHex(c, 4) + ', Y';
         end;
    $5D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'EOR $' + IntToHex(c, 4) + ', X';
         end;
    $46: begin
           system.inc(addr);
           s := 'LSR $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $4E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LSR $' + IntToHex(c, 4);
         end;
    $56: begin
           system.inc(addr);
           s := 'LSR $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $5E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LSR $' + IntToHex(c, 4) + ', X';
         end;
    $48: s := 'PHA';
    $4A: s := 'LSR A';
    $4C: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'JMP $' + IntToHex(c, 4);
         end;
    $6C: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'JMP ($' + IntToHex(c, 4) + ')';
         end;
    $7C: s := 'JMP';
    $50: begin
           system.inc(addr);
           s := 'BVC ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $58: s := 'CLI';
    $5A: s := 'PHY';
    $60: s := 'RTS';
    $61,
    $65: begin
           system.inc(addr);
           s := 'ADC ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $69: begin
           system.inc(addr);
           s := 'ADC #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $6D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ADC ' + IntToHex(c, 4);
         end;
    $71,
    $72,
    $75: begin
           system.inc(addr);
           s := 'ADC ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $79,
    $7D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ADC ' + IntToHex(c, 4) + ', X';
         end;
    $64,
    $74,
    $9C,
    $9E: s := 'STZ';
    $66: begin
           system.inc(addr);
           s := 'ROR $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $6E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ROR $' + IntToHex(c, 4);
         end;
    $76: begin
           system.inc(addr);
           s := 'ROR $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $7E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'ROR $' + IntToHex(c, 4) + ', X';
         end;
    $68: s := 'PLA';
    $6A: s := 'ROR A';
    $70: begin
           system.inc(addr);
           s := 'BVS ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $78: s := 'SEI';
    $7A: s := 'PLY';
    $80: s := 'BRA';
    $81: begin
           system.inc(addr);
           s := 'STA ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
         end;
    $85: begin
           system.inc(addr);
           s := 'STA $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $8D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'STA $' + IntToHex(c, 4);
         end;
    $91: begin
           system.inc(addr);
           s := 'STA ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $92,
    $95: begin
           system.inc(addr);
           s := 'STA $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $99: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'STA $' + IntToHex(c, 4) + ', Y';
         end;
    $9D: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'STA $' + IntToHex(c, 4) + ', X';
         end;
    $84: begin
           system.inc(addr);
           s := 'STY $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $8C: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'STY $' + IntToHex(c, 4);
         end;
    $94: begin
           system.inc(addr);
           s := 'STY $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $86: begin
           system.inc(addr);
           s := 'STX $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $8E: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'STX $' + IntToHex(c, 4);
         end;
    $96: begin
           system.inc(addr);
           s := 'STX $' + IntToHex(MemoryReadOpcode(addr), 2) + ', Y';
         end;
    $88: s := 'DEY';
    $8A: s := 'TXA';
    $90: begin
           system.inc(addr);
           s := 'BCC ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $98: s := 'TYA';
    $9A: s := 'TXS';
    $A0: begin
           system.inc(addr);
           s := 'LDY #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $A4: begin
           system.inc(addr);
           s := 'LDY $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $AC: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LDY $' + IntToHex(c, 4);
         end;
    $B4: begin
           system.inc(addr);
           s := 'LDY $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $BC: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LDY $' + IntToHex(c, 4) + ', X';
         end;
    $A1: begin
           system.inc(addr);
           s := 'LDA ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
         end;
    $A5: begin
           system.inc(addr);
           s := 'LDA $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $A9: begin
           system.inc(addr);
           s := 'LDA #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $AD: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LDA $' + IntToHex(c, 4);
         end;
    $B1: begin
           system.inc(addr);
           s := 'LDA ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $B2,
    $B5: begin
           system.inc(addr);
           s := 'LDA $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $B9: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LDA $' + IntToHex(c, 4) + ', Y';
         end;
    $BD: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LDA $' + IntToHex(c, 4) + ', X';
         end;
    $A2: begin
           system.inc(addr);
           s := 'LDX #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $A6: begin
           system.inc(addr);
           s := 'LDX $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $AE: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LDX $' + IntToHex(c, 4);
         end;
    $B6: begin
           system.inc(addr);
           s := 'LDX $' + IntToHex(MemoryReadOpcode(addr), 2) + ', Y';
         end;
    $BE: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'LDX $' + IntToHex(c, 4) + ', Y';
         end;
    $A8: s := 'TAY';
    $AA: s := 'TAX';
    $B0: s := 'BCS';
    $B8: s := 'CLV';
    $BA: s := 'TSX';
    $C0: begin
           system.inc(addr);
           s := 'CPY #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $C4: begin
           system.inc(addr);
           s := 'CPY ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $CC: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'CPY ' + IntToHex(c, 4);
         end;
    $C1: begin
           system.inc(addr);
           s := 'CMP (' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $C5: begin
           system.inc(addr);
           s := 'CMP ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $C9: begin
           system.inc(addr);
           s := 'CMP #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $CD: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'CMP ' + IntToHex(c, 4);
         end;
    $D1: begin
           system.inc(addr);
           s := 'CMP (' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $D2,
    $D5: begin
           system.inc(addr);
           s := 'CMP ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $D9: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'CMP ' + IntToHex(c, 4) + ', Y';
         end;
    $DD: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'CMP ' + IntToHex(c, 4) + ', X';
         end;
    $C6: begin
           system.inc(addr);
           s := 'DEC ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $CE: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'DEC ' + IntToHex(c, 4);
         end;
    $D6: begin
           system.inc(addr);
           s := 'DEC ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $DE: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'DEC ' + IntToHex(c, 4) + ', X';
         end;
    $C8: s := 'INY';
    $CA: s := 'DEX';
    $D0: begin
           system.inc(addr);
           s := 'BNE ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $D8: s := 'CLD';
    $DA: s := 'PHX';
    $E0: begin
           system.inc(addr);
           s := 'CPX #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $E4: begin
           system.inc(addr);
           s := 'CPX ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $EC: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'CPX ' + IntToHex(c, 4);
         end;
    $E1: begin
           system.inc(addr);
           s := 'SBC ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
         end;
    $E5: begin
           system.inc(addr);
           s := 'SBC $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $E9: begin
           system.inc(addr);
           s := 'SBC #' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $ED: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'SBC $' + IntToHex(c, 4);
         end;
    $F1: begin
           system.inc(addr);
           s := 'SBC ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
         end;
    $F2,
    $F5: begin
           system.inc(addr);
           s := 'SBC $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $F9: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'SBC $' + IntToHex(c, 4) + ', Y';
         end;
    $FD: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'SBC $' + IntToHex(c, 4) + ', X';
         end;
    $E6: begin
           system.inc(addr);
           s := 'INC $' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $EE: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'INC $' + IntToHex(c, 4);
         end;
    $F6: begin
           system.inc(addr);
           s := 'INC $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
         end;
    $FE: begin
           system.inc(addr, 2);
           c := MemoryReadWord(addr-1);
           s := 'INC $' + IntToHex(c, 4) + ', X';
         end;
    $E8: s := 'INX';
    $F0: begin
           system.inc(addr);
           s := 'BEQ ' + IntToHex(MemoryReadOpcode(addr), 2);
         end;
    $F8: s := 'SED';
    $FA: s := 'PLX';
   end;
   system.inc(addr);
 end;
 Result := s;
end; { DAsm }


end.
